(document:surround "/std/base")
(document:insert "/std/functions")

(document:envelop with-translation _ "alterator-net-wifi")

width 500
height 350

(gridbox
 columns "10;20;60;10"
 (spacer)
 (label (bold (_ "Status")))
 (document:id status-field (label ""))
 (spacer)
 
 (spacer)
 (label (bold (_ "Access point")))
 (document:id bssid-field (label ""))
 (spacer)
 
 (spacer)
 (document:id reconfigure-button (button (_ "Reconfigure") align "left"))
 (spacer)
 (spacer))


(gridbox
 columns "10;80;10"
 
 (spacer)
 (label (bold (_ "Configured networks")))
 (spacer)
 
 (spacer)
 (document:id
  networks-field
  (listbox
   columns 3
   header (vector (_ "Network name") (_ "Access point") (_ "Flags"))))
 (spacer)
 
 (spacer)
 (hbox align "left"
       (document:id new-button (button "+"))
       (document:id del-button (button "-"))
       (document:id edit-button (button (_ "Edit")))
       (document:id select-button (button (_ "Select"))))
 (spacer))

(document:id quit-button (button (_ "Quit") align "left"))


;;; functions

(define *interface* (string-append "/net-wifi/" (global 'interface)))

(define (read-status)
  (let ((data (woo-read-first *interface*)))
    (status-field text (woo-get-option data 'status))
    (bssid-field text (woo-get-option data 'bssid))))

(define (reconfigure)
  (woo-write *interface* 'reconfigure #t))

(define *ids* (make-cell '()))

(define (networks)
  (let ((lst (woo-list (string-append *interface* "/networks"))))
    (cell-set! *ids*
               (map (lambda(x)
                      (woo-get-option x 'name))
                    lst))
    (networks-field
     rows
     (map (lambda(item)
            (vector (woo-get-option item 'essid)
                    (woo-get-option item 'bssid)
                    (woo-get-option item 'flags)))
          lst))))

(define (network-current)
  (let ((cur (networks-field current)))
    (and (>= cur 0)
         (list-ref (cell-ref *ids*) cur))))

(define (message-handler args)
  (and (eq? (car args) 'wpa-supplicant)
       (read-status)))

(define (network-popup action)
   (document:popup "/net-wifi/network"
                   'interface (global 'interface)
                   'id action))

(define (network-new)
  (network-popup 'new))

(define (network-edit)
  (let ((cur (network-current)))
    (and cur (network-popup cur))))

(define (network-delete)
  (woo-catch/message
   (thunk
    (let ((cur (network-current)))
      (and cur
           (eq? 'yes (document:popup-warning (_ "Are you really want to delete network?") 'no 'yes))
            (woo-try "remove" (string-append *interface* "/networks/" cur)))))))

(define (network-select)
  (woo-catch/message
   (thunk
    (let ((cur (network-current)))
      (and cur
           (eq? 'yes (document:popup-warning (_ "Are you really want to change current network?") 'no 'yes))
           (woo-try "select" (string-append *interface* "/networks/" cur)))))))

;;; logic

(document:root (when loaded
                 (woo-try "monitor" "/net-wifi" 'mode "on") ;;init watch timer
                 (networks)
		 (add-mailbox message-handler)))

(quit-button (when clicked
               (woo-try "monitor" "/net-wifi" 'mode "off")
	       (remove-mailbox message-handler)
               (document:end)))

(reconfigure-button (when clicked
                      (reconfigure)))

(new-button (when clicked
              (and (network-new)
                   (networks))))

(edit-button (when clicked
               (and (network-edit)
                    (networks))))

(del-button (when clicked
              (and (network-delete)
                   (networks))))

(select-button (when clicked
                 (and (network-select)
                      (networks))))
