(document:surround "/std/base")
(document:insert "/std/functions")

(document:envelop with-translation _ "alterator-net-wifi")

width 550
height 450

(gridbox
 columns "5;90;5"
 
 (spacer)
 (label (bold (_ "Available networks")))
 (spacer)
 
 (spacer)
 (document:id
  scan-results-field
  (listbox
   height 70
   columns 3
   header (vector (_ "Network name") (_ "Access point") (_ "Flags"))))
 (spacer)
 
 (spacer)
 (hbox align "left"
   (document:id scan-button (button (_ "Scan") align "right"))
   (document:id status-field (label "" align "left")))
 (spacer))


(gridbox
 columns "5;30;60;5"

 (spacer)
 (label (bold (_ "Network name")))
 (document:id essid-field (edit "" widget-name "essid"))
 (spacer)

 (spacer)
 (label (bold (_ "Security")))
 (document:id auth_type-field (combobox widget-name "auth_type"))
 (spacer)

 (spacer)
 (label (bold (_ "User name")))
 (document:id username-field (edit "" widget-name "username"))
 (spacer)

 (spacer)
 (label (bold (_ "Network key (use 0x for hex)")))
 (document:id key-field (edit "" widget-name "key"))
 (spacer)

 (spacer)
 (label (bold (_ "Encryption")))
 (document:id pairwise-field (combobox widget-name "pairwise"))
 (spacer)

 (spacer)
 (label (bold (_ "EAP method")))
 (document:id eap-field (combobox widget-name "eap"))
 (spacer))

(hbox
 (document:id ok-button (button (_ "OK")))
 (document:id cancel-button (button (_ "Cancel"))))

;;; functions
(define (name+label x)
  (let ((name (woo-get-option x 'name)))
    (cons name (woo-get-option x 'label name))))

(define (make-variants x f y)
  (let ((lst (map name+label (woo-list (string-append *interface* "/" y)))))
    (cell-set! x lst)
    (f rows (map cdr lst))))

(define (get-variant x f)
  (let ((cur (f current)))
    (if (>= cur 0)
        (car (list-ref (cell-ref x) cur))
        "")))
(define (set-variant x f v)
  (f current (list-index (lambda(x) (string=? (car x) v))
                         (cell-ref x))))

(define *interface* (string-append "/net-wifi/" (global 'interface)))
(define *auth-types* (make-cell '()))
(define *eap-methods* (make-cell '()))
(define *pairwise-types* (make-cell '()))

(define (network-params)
  (list
   'essid (essid-field text)
   'auth_type (get-variant *auth-types* auth_type-field)
   'username (username-field text)
   'key (key-field text)
   'eap (get-variant *eap-methods* eap-field)
   'pairwise (get-variant *pairwise-types* pairwise-field)))

(define (network-new)
  (woo-catch/message
   (thunk
    (apply woo-new *interface* (network-params)))))

(define (network-edit id)
  (woo-catch/message
   (thunk
    (apply woo-write
           (string-append *interface* "/networks/" id)
           (network-params)))))

(define (network-read id)
  (let ((data (woo-read-first (string-append *interface* "/networks/" id))))
    (essid-field text (woo-get-option data 'essid))
    (key-field text (woo-get-option data 'key))
    (username-field text (woo-get-option data 'username))
    (set-variant *auth-types* auth_type-field (woo-get-option data 'auth_type))
    (set-variant *eap-methods* eap-field (woo-get-option data 'eap))
    (set-variant *pairwise-types* pairwise-field (woo-get-option data 'pairwise))))

(define (small-comment x)
  (string-append "(<small>" x "</small>)"))

(define (read-status)
  (let ((data (woo-read-first *interface*)))
    (status-field text (small-comment (woo-get-option data 'status)))))

(define (scan)
  (woo-write *interface* 'scan #t)
  (message-handler '(wpa-supplicant)))

(define (scan-results)
  (scan-results-field
   rows
   (map (lambda(item)
          (vector (woo-get-option item 'name)
                  (woo-get-option item 'bssid)
                  (woo-get-option item 'flags)))
        (woo-list (string-append *interface* "/scan_results")))))

(scan-button (when clicked
               (scan)))

(define (message-handler args)
  (and (eq? (car args) 'wpa-supplicant)
       (begin (read-status)
	      (scan-results))))

(define (process-excludes value)
  (cond
   ((string=? value "nopasswd")
    ((widgets key-field
              username-field
              eap-field
              pairwise-field) activity #f))
   ((string=? value "wep")
    ((widgets username-field
              eap-field
              pairwise-field) activity #f)
    ((widgets key-field) activity #t))
   ((member value '("wpa-psk" "wpa2-psk"))
    ((widgets username-field
              eap-field) activity #f)
    ((widgets key-field
              pairwise-field) activity #t))
   ((member value '("wpa-eap" "wpa2-eap"))
    ((widgets key-field
              username-field
              eap-field
              pairwise-field) activity #t))))

;;; logic

(document:root (when loaded
                 (auth_type-field (when selected
                                    (process-excludes (get-variant *auth-types* auth_type-field))))
                 
                 (woo-catch/message
                  (thunk
                   (make-variants *auth-types* auth_type-field "/auth_types")
                   (make-variants *eap-methods* eap-field "/eap_methods")
                   (make-variants *pairwise-types* pairwise-field "/pairwise_types")

                   (if (string? (global 'id)) 
		       (network-read (global 'id))
		       (auth_type-field current 0))
                   (auth_type-field selected)
                   
                   (scan-results)
                   (add-mailbox message-handler)))))
  
(cancel-button (when clicked
                 (remove-mailbox message-handler)
                 (document:end #f)))

(ok-button (when clicked
             (remove-mailbox message-handler)
             (if (eq? (global 'id) 'new)
                 (network-new)
                 (network-edit (global 'id)))
             (document:end #t)))
